{% extends frame.html %}

{% block body %}
<h4 style="margin-left: 1em;">Dashboard</h4>
<ul>
    <li><a href="/aegis/reports">Reports</a></li>
    <li><a href="/aegis/hydra">Hydra</a></li>
</ul>
{% end %}
